//**************************************************************************//
//																			//
// File:		Host Port Access.c											//
//																			//
// Project:		ADSP-2191 Host Port booting									//
//																			//
// Description: This file contains functions handling the low level access	//
//				to the Target Host Port via the PLD. They have to be called	//
//				with parameters according to the access type (Target		//
//				address, value, or both).									//
//				The following functions are provided:						//
//																			//
//				   Write_Target_IO_Direct									//
//				   Write_Target_DM_Direct									//
//				   Write_Target_PM_Direct									//
//				   Read_Target_IO_Direct									//
//				   Read_Target_DM_Direct									//
//				   Read_Target_PM_Direct									//
//				   Write_Target_DM_DMA										//
//				   Write_Target_PM_DMA										//
//				   Get_PLD_Version											//
//				   Init_Target												//
//																			//
// Note:		1. The functions in this file need to be called with		//
//				   address parameters as seen on the DSP!					//
//				   The Host Port address mapping, as explained in the		//
//				   ADSP-2191 Hardware Reference (addresses are shifted left	//
//				   by 1 or 2 bit), is done in the PLD!						//
//																			//
//				2. Register HPPR on the Target is NOT modified by these		//
//				   functions, and is expected to be set correctly for the	//
//				   access to be performed!									//
//				   On the Target, set:										//
//				      Bit DTYPE in register HPPR to 0 for 16-bit accesses	//
//				      Bit DTYPE in register HPPR to 1 for 24-bit accesses	//
//																			//
// Examples:	For writes in Direct Mode, an address and a value are		//
//				transferred to the Target Host Port.To transfer the value	//
//				0x1234 in Direct Mode to memory location 0x0002 in Data		//
//				Memory, the following function call would be used:			//
//																			//
//					Write_Target_DM_Direct(0x0002, 0x1234);					//
//																			//
//				For reads in Direct Mode, only an address is sent to the	//
//				Target. The following instruction uses Direct Mode to read	//
//				the value from address 0x0004 in Program Memory and returns	//
//				the value to the caller (variable is of type unsigned long)://
//																			//
//					variable = Read_Target_PM_Direct(0x0004);				//
//																			//
//				For writes in DMA Mode, only a value (no address) is sent	//
//				via the Target Host Port. To transfer the value 0x123456 in	//
//				DMA Mode to the Target Host Port, the following function	//
//				call can be used (requires preceding configuration of Host	//
//				Port DMA!):													//
//																			//
//					Write_Target_PM_DMA(0x123456);							//
//																			//
//**************************************************************************//
//																			//
// Additional Information (not required for using these functions):			//
//				Since the Host Port on the Target uses more signals than	//
//				the External Port on the Host, a "virtual address space" is	//
//				used for transferring the additional information to the PLD.//
//				The following table shows the mapping of possible Host Port	//
//				transactions to addresses in the PLD. However, these		//
//				addresses are not physically implemented in the design.		//
//				Rather are the information encoded in the address lines		//
//				used to perform the appropriate Host Port transfer.			//
//				(When reading from address 8 in the PLD, the revision		//
//				number of the VHDL design is returned.)						//
//																			//
//				   PLD Address  |   Access cycle    |  Read/Write			//
//				 -------------------------------------------------			//
//				        0       |   HP DM ADDRESS   |      W				//
//				        1       |   HP DM DATA      |     R/W				//
//				        2       |   HP PM ADDRESS   |      W				//
//				        3       |   HP PM DATA      |     R/W				//
//				        4       |   HP IO ADDRESS   |      W				//
//				        5       |   HP IO DATA      |     R/W				//
//				        6       |   Reserved        |      -				//
//				        7       |   Reserved        |      -				//
//				        8       |   REG_VERSION     |      R				//
//				      9 - 15    |   Reserved        |      -				//
//																			//
//**************************************************************************//

#include "Host Port Access.h"

//--------------------------------------------------------------------------//
// Function:	Write_Target_IO_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of IO space location							//
//				ui_Value:													//
//					Value to be written to IO space location				//
//																			//
// Return:		None														//
//																			//
// Description:	This function performs one 16-bit write access to IO space	//
//				via Host Port in Direct Mode, which consists of one Address	//
//				write cycle and one Data write cycle.						//
//--------------------------------------------------------------------------//
void Write_Target_IO_Direct(const unsigned int ui_Address, const unsigned int ui_Value)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_IO_ADDRESS, ui_Address);	// address cycle of HP access
	io_space_write(HP_IO_DATA, ui_Value);		// data cycle of HP access
}

//--------------------------------------------------------------------------//
// Function:	Read_Target_IO_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of IO space location							//
//																			//
// Return:		Value read from IO space									//
//																			//
// Description:	This function performs one 16-bit read access from IO space	//
//				via Host Port in Direct Mode, which consists of one Address	//
//				write cycle and one Data read cycle.						//
//--------------------------------------------------------------------------//
unsigned int Read_Target_IO_Direct(const unsigned int ui_Address)
{
	unsigned int ui_Value;

	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_IO_ADDRESS, ui_Address);	// address cycle of HP access
	ui_Value = io_space_read(HP_IO_DATA);		// data cycle of HP access

	return(ui_Value);
}

//--------------------------------------------------------------------------//
// Function:	Write_Target_DM_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of DM location									//
//				ui_Value:													//
//					Value to be written to DM location						//
//																			//
// Return:		None														//
//																			//
// Description:	This function performs one 16-bit write access to DM via	//
//				Host Port in Direct Mode, which consists of one Address		//
//				write cycle and one Data write cycle.						//
//																			//
// Note:		Bit DTYPE in register HPPR on the Target is expected to be	//
//				0 for this access.											//
//--------------------------------------------------------------------------//
void Write_Target_DM_Direct(const unsigned int ui_Address, const unsigned int ui_Value)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_DM_ADDRESS, ui_Address);	// address cycle of HP access
	io_space_write(HP_DM_DATA, ui_Value);		// data cycle of HP access
}

//--------------------------------------------------------------------------//
// Function:	Read_Target_DM_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of DM location									//
//																			//
// Return:		Value read from DM											//
//																			//
// Description:	This function performs one 16-bit read access from DM via	//
//				Host Port in Direct Mode, which consists of one Address		//
//				write cycle and one Data read cycle.						//
//																			//
// Note:		Bit DTYPE in register HPPR on the Target is expected to be	//
//				0 for this access.											//
//--------------------------------------------------------------------------//
unsigned int Read_Target_DM_Direct(const unsigned int ui_Address)
{
	unsigned int ui_Value;

	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_DM_ADDRESS, ui_Address);	// address cycle of HP access
	ui_Value = io_space_read(HP_DM_DATA);		// data cycle of HP access

	return(ui_Value);
}

//--------------------------------------------------------------------------//
// Function:	Write_Target_PM_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of PM location									//
//				ul_Value:													//
//					Value to be written to PM location						//
//																			//
// Return:		None														//
//																			//
// Description:	This function performs one 24-bit write access to PM via	//
//				Host Port in Direct Mode, which consists of one Address		//
//				write cycle and two Data write cycles.						//
//																			//
// Note:		Bit DTYPE in register HPPR on the Target is expected to be	//
//				1 for this access.											//
//--------------------------------------------------------------------------//
void Write_Target_PM_Direct(const unsigned int ui_Address, const unsigned long ul_Value)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_PM_ADDRESS, ui_Address);	// address cycle of HP access
	io_space_write(HP_PM_DATA, ul_Value << 8);	// first data cycle of HP access
	io_space_write(HP_PM_DATA, ul_Value >> 8);	// second data cycle of HP access
}

//--------------------------------------------------------------------------//
// Function:	Read_Target_PM_Direct										//
//																			//
// Parameters:	ui_Address:													//
//					Address of PM location									//
//																			//
// Return:		Value read from PM											//
//																			//
// Description:	This function performs one 24-bit read access from PM via	//
//				Host Port in Direct Mode, which consists of one Address		//
//				write cycle and two Data read cycles.						//
//																			//
// Note:		Bit DTYPE in register HPPR on the Target is expected to be	//
//				1 for this access.											//
//--------------------------------------------------------------------------//
unsigned long Read_Target_PM_Direct(const unsigned int ui_Address)
{
	unsigned long ul_Value;

	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_PM_ADDRESS, ui_Address);	// address cycle of HP access
	ul_Value = (unsigned long)(io_space_read(HP_PM_DATA) >> 8) & 0x000000ff;
												// first data cycle of HP access
	ul_Value = ul_Value | (((unsigned long)io_space_read(HP_PM_DATA) << 8) & 0x00ffff00);
												// second data cycle of HP access

	return(ul_Value);
}

//--------------------------------------------------------------------------//
// Function:	Write_Target_DM_DMA											//
//																			//
// Parameters:	ui_Value:													//
//					Value to be transferred to DM location					//
//																			//
// Return:		None														//
//																			//
// Description:	This function performs one 16-bit write access to DM via	//
//				Host Port in DMA Mode, which consists of one Data write		//
//				cycle (no Address cycle).									//
//--------------------------------------------------------------------------//
void Write_Target_DM_DMA(const unsigned int ui_Value)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_DM_DATA, ui_Value);		// data cycle of HP access
}

//--------------------------------------------------------------------------//
// Function:	Write_Target_PM_DMA											//
//																			//
// Parameters:	ul_Value:													//
//					Value to be transferred to PM location					//
//																			//
// Return:		None														//
//																			//
// Description:	This function performs one 24-bit write access to PM via	//
//				Host Port in DMA Mode, which consists of two Data write		//
//				cycles (no Address cycle).									//
//--------------------------------------------------------------------------//
void Write_Target_PM_DMA(const unsigned long ul_Value)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_PM_DATA, ul_Value << 8);	// first data cycle of HP access
	io_space_write(HP_PM_DATA, ul_Value >> 8);	// second data cycle of HP access
}

//--------------------------------------------------------------------------//
// Function:	Get_PLD_Version												//
//																			//
// Parameters:	None														//
//																			//
// Return:		Version number of PLD										//
//																			//
// Description:	This reads the version number of the PLD.					//
//--------------------------------------------------------------------------//
unsigned int Get_PLD_Version(void)
{
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	return(io_space_read(REG_VERSION));
}

//--------------------------------------------------------------------------//
// Function:	Init_Target													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description: This function configures the Host Port on the Target DSP	//
//				for 16-bit mode by writing the value 0x0f01 into register	//
//				HPCR (0x1c01). Since the default configuration after reset	//
//				is 8-bit mode, two Data cycles are necessary for this first	//
//				write to HPCR.												//
//--------------------------------------------------------------------------//
void Init_Target(void)
{
	// configure HPCR on Target (two 8-bit accesses necessary)
	sysreg_write(sysreg_IOPG, EXTERNAL_IO_PAGE);
	io_space_write(HP_IO_ADDRESS, 0x1c01);		// address cycle of HP access
	io_space_write(HP_IO_DATA, 0x01);			// first Data byte of HPCR
	io_space_write(HP_IO_DATA, 0x0f);			// second Data byte of HPCR

	// configure HPPR on Target
	Write_Target_IO_Direct(0x1c02, 0x0000);
}

