---------------------------------------------------------------------------------------------------
--
-- Name:		Watchdog module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	watchdog.vhd
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		i_clock
--					Clock for watchdog timer
--				i_reset
--					Reset for watchdog timer
--				i_start
--					Start signal for watchdog timer
--
-- Outputs:		o_error
--					Signal indicating a timer overflow, or a restart while the watchdog timer was
--					still running
--
-- Description:	This module implements a watchdog timer that is started by asserting input signal
--				i_start. If input i_reset is asserted before the counter expires, the count
--				process stops. If the counter expires, or if a start signal is detected before a
--				reset signal then output o_error gets asserted for one clock cycle.
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE work.pack_const.ALL;

-- entity for watchdog
ENTITY watchdog IS
	PORT(
		i_clock				: IN	STD_LOGIC;	-- watchdog runs at system clock
		i_reset				: IN	STD_LOGIC;	-- asynchronous reset
		i_start				: IN	STD_LOGIC;	-- start signal for watchdog
		o_error				: OUT	STD_LOGIC	-- counter overflow, or restart before reset
	);
END watchdog;

-- architecture for watchdog timer
ARCHITECTURE watchdog OF watchdog IS
	SIGNAL wd_count			: STD_LOGIC;		-- watchdog counts while this signal is asserted
	SIGNAL wd_expired		: STD_LOGIC;		-- error signal for time out
	SIGNAL wd_restart		: STD_LOGIC;		-- error signal for restart before reset

BEGIN
	-- signal error after watchdog time out or two start signals in sequence
	o_error <= wd_expired OR wd_restart;

	wd_counter: PROCESS (i_reset, i_clock)
		VARIABLE counter_value: INTEGER RANGE 0 TO MAX_WD_VALUE;
	BEGIN
		-- reset all registered values
		IF (i_reset = ASSERTED) THEN
			wd_count		<= DEASSERTED;
			wd_restart		<= DEASSERTED;
			wd_expired		<= DEASSERTED;
			counter_value	:= 0;
		-- check for risging edge of clock
		ELSIF (i_clock'EVENT AND i_clock = ASSERTED) THEN
			-- reset error signals by default
			wd_expired <= DEASSERTED;
			wd_restart <= DEASSERTED;
			-- enable activation signal for counter on detection of start pulse
			IF (i_start = ASSERTED) THEN
				wd_count <= ASSERTED;
			END IF;
			-- increment counter if activation signal is asserted
			IF (wd_count = ASSERTED) THEN
				counter_value := counter_value + 1;
				-- signal counter overflow and reset counter when maximum value is reached
				IF (counter_value = MAX_WD_VALUE) THEN
					wd_expired		<= ASSERTED;
					counter_value	:= 0;
				END IF;
				-- assert error signal when restart is detected while counter is still running
				IF (i_start = ASSERTED) THEN
					wd_restart <= ASSERTED;
				END IF;
			END IF;
		END IF;
	END PROCESS wd_counter;
END watchdog;
